<?php

namespace App\Command\Migration;

use App\Entity\HardwareProfile;
use App\Entity\OrganizationalUnit;
use Doctrine\ORM\EntityManagerInterface;
use Doctrine\ORM\Query\ResultSetMapping;
use Doctrine\Persistence\ManagerRegistry;
use Symfony\Component\Console\Attribute\AsCommand;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;

#[AsCommand(name: 'opengnsys:migrate-hardware-profile', description: 'Migrate hardware and hardware profile data')]
class MigrateHardwareAndHardwareProfileCommand extends Command
{
    public function __construct(
        private readonly EntityManagerInterface $entityManager,
        private readonly ManagerRegistry $doctrine
    )
    {
        parent::__construct();
    }


    protected function execute(InputInterface $input, OutputInterface $output): int
    {
        /** @var EntityManagerInterface $oldDatabaseEntityManager */
        $oldDatabaseEntityManager = $this->doctrine->getManager('og_1');

        $organizationalUnitRepository = $this->entityManager->getRepository(OrganizationalUnit::class);
        $hardwareProfileRepository = $this->entityManager->getRepository(HardwareProfile::class);

        /** Obtener los centros de la base de datos antigua **/
        $rsmHardwareProfiles = new ResultSetMapping();
        $rsmHardwareProfiles->addScalarResult('idperfilhard', 'idperfilhard');
        $rsmHardwareProfiles->addScalarResult('descripcion', 'descripcion');
        $rsmHardwareProfiles->addScalarResult('comentarios', 'perfilhard.comentarios');
        $rsmHardwareProfiles->addScalarResult('grupoid', 'perfilhard.grupoid');
        $rsmHardwareProfiles->addScalarResult('idcentro', 'perfilhard.idcentro');

        $hardwareProfilesQuery = $oldDatabaseEntityManager->createNativeQuery('SELECT idperfilhard, descripcion, grupos.comentarios, perfileshard.grupoid, perfileshard.idcentro FROM perfileshard LEFT JOIN grupos ON perfileshard.grupoid = grupos.idgrupo', $rsmHardwareProfiles);
        $hardwareProfiles = $hardwareProfilesQuery->getResult();

        /** Perfiles hardware **/
        $output->writeln("PERFILES HARDWARE TOTAL: ". count($hardwareProfiles));
        foreach ($hardwareProfiles as $hardwareProfile){
            $hardwareProfileEntity = null;
            $hardwareProfileEntity = $hardwareProfileRepository->findOneBy(['migrationId' => $hardwareProfile['idperfilhard']]);
            if(!$hardwareProfileEntity){
                $hardwareProfileEntity = new HardwareProfile();
                $hardwareProfileEntity->setMigrationId($hardwareProfile['idperfilhard']);
                $hardwareProfileEntity->setDescription($hardwareProfile['descripcion']);
                $hardwareProfileEntity->setComments($hardwareProfile['perfilhard.comentarios']);
            }

            $migrationId = $hardwareProfile['perfilhard.grupoid'] === 0 ? $hardwareProfile['perfilhard.idcentro'] : $hardwareProfile['perfilhard.grupoid'];
            $organizationalUnit = $organizationalUnitRepository->findOneBy(['migrationId' => $migrationId]);

            if ($organizationalUnit){
                $hardwareProfileEntity->setOrganizationalUnit($organizationalUnit);
            }

            $this->entityManager->persist($hardwareProfileEntity);
        }
        $this->entityManager->flush();

        return Command::SUCCESS;
    }
}
