<?php

namespace App\Dto\Input;

use App\Entity\HardwareProfile;
use App\Entity\Menu;
use App\Entity\NetworkSettings;
use App\Validator\Constraints\OrganizationalUnitMulticastMode;
use App\Validator\Constraints\OrganizationalUnitMulticastPort;
use App\Validator\Constraints\OrganizationalUnitP2PMode;
use Symfony\Component\Serializer\Annotation\Groups;
use Symfony\Component\Validator\Constraints as Assert;

class NetworkSettingsInput
{
    #[Groups(['organizational-unit:write'])]
    public ?string $proxy = null;

    #[Groups(['organizational-unit:write'])]
    public ?string $dns = null;

    #[Groups(['organizational-unit:write'])]
    public ?string $netmask = null;

    #[Groups(['organizational-unit:write'])]
    public ?string $router = null;

    #[Groups(['organizational-unit:write'])]
    public ?string $ntp = null;

    #[OrganizationalUnitP2PMode]
    #[Groups(['organizational-unit:write'])]
    public ?string $p2pMode = null;

    #[Groups(['organizational-unit:write'])]
    public ?int $p2pTime = null;

    #[Assert\Ip()]
    #[Groups(['organizational-unit:write'])]
    public ?string $mcastIp = null;

    #[Groups(['organizational-write:write'])]
    public ?int $mcastSpeed = null;

    #[OrganizationalUnitMulticastPort]
    #[Groups(['organizational-unit:write'])]
    public ?int $mcastPort = null;

    #[OrganizationalUnitMulticastMode]
    #[Groups(['organizational-unit:write'])]
    public ?string $mcastMode = null;

    #[Groups(['organizational-unit:write'])]
    public ?Menu $menu = null;

    #[Groups(['organizational-unit:write'])]
    public ?HardwareProfile $hardwareProfile = null;

    #[Groups(['organizational-unit:write'])]
    public ?bool $validation = null;

    public function __construct(?NetworkSettings $networkSettings = null)
    {
        if (!$networkSettings) {
            return;
        }

        $this->proxy = $networkSettings->getProxy();
        $this->dns = $networkSettings->getDns();
        $this->netmask = $networkSettings->getNetmask();
        $this->router = $networkSettings->getRouter();
        $this->ntp = $networkSettings->getNtp();
        $this->p2pMode = $networkSettings->getP2pMode();
        $this->p2pTime = $networkSettings->getP2pTime();
        $this->mcastIp = $networkSettings->getMcastIp();
        $this->mcastSpeed = $networkSettings->getMcastSpeed();
        $this->mcastPort = $networkSettings->getMcastPort();
        $this->mcastMode = $networkSettings->getMcastMode();
        $this->menu = $networkSettings->getMenu();
        $this->hardwareProfile = $networkSettings->getHardwareProfile();
        $this->validation = $networkSettings->getValidation();
    }

    public function createOrUpdateEntity(?NetworkSettings $networkSettings = null): NetworkSettings
    {
        if (!$networkSettings) {
            $networkSettings = new NetworkSettings();
        }

        $networkSettings->setProxy($this->proxy);
        $networkSettings->setDns($this->dns);
        $networkSettings->setNetmask($this->netmask);
        $networkSettings->setRouter($this->router);
        $networkSettings->setNtp($this->ntp);
        $networkSettings->setP2pMode($this->p2pMode);
        $networkSettings->setP2pTime($this->p2pTime);
        $networkSettings->setMcastIp($this->mcastIp);
        $networkSettings->setMcastSpeed($this->mcastSpeed);
        $networkSettings->setMcastPort($this->mcastPort);
        $networkSettings->setMcastMode($this->mcastMode);
        $networkSettings->setMenu($this->menu);
        $networkSettings->setHardwareProfile($this->hardwareProfile);
        $networkSettings->setValidation($this->validation);

        return $networkSettings;
    }
}