<?php

namespace App\Dto\Input;

use ApiPlatform\Metadata\ApiProperty;
use App\Dto\Output\OrganizationalUnitOutput;
use App\Entity\OrganizationalUnit;
use App\Validator\Constraints\OrganizationalUnitMulticastMode;
use App\Validator\Constraints\OrganizationalUnitMulticastPort;
use App\Validator\Constraints\OrganizationalUnitP2PMode;
use App\Validator\Constraints\OrganizationalUnitType;
use Symfony\Component\Serializer\Annotation\Groups;
use Symfony\Component\Validator\Constraints as Assert;

class OrganizationalUnitInput
{
    #[Assert\NotBlank]
    #[Groups(['organizational-unit:write'])]
    public ?string $name = null;

    #[Groups(['organizational-unit:write'])]
    public ?OrganizationalUnitOutput $parent = null;

    #[Groups(['organizational-unit:write'])]
    public ?string $description = null;

    #[Groups(['organizational-unit:write'])]
    public ?string $comments = null;

    #[Groups(['organizational-unit:write'])]
    #[OrganizationalUnitType]
    public ?string $type = null;

    #[Groups(['organizational-unit:write'])]
    #[Assert\Valid()]
    public ?NetworkSettingsInput $networkSettings = null;

    public function __construct(?OrganizationalUnit $organizationalUnit = null)
    {
        if (!$organizationalUnit) {
            return;
        }

        $this->name = $organizationalUnit->getName();
        if ($organizationalUnit->getParent()) {
            $this->parent = new OrganizationalUnitOutput($organizationalUnit->getParent());
        }
        $this->description = $organizationalUnit->getDescription();
        $this->comments = $organizationalUnit->getComments();
        $this->type = $organizationalUnit->getType();
        if ($organizationalUnit->getNetworkSettings()){
            $this->networkSettings = new NetworkSettingsInput($organizationalUnit->getNetworkSettings());
        }
    }

    public function createOrUpdateEntity(?OrganizationalUnit $organizationalUnit = null): OrganizationalUnit
    {
        if (!$organizationalUnit) {
            $organizationalUnit = new OrganizationalUnit();
        }

        $organizationalUnit->setName($this->name);
        if ($this->parent) {
            $organizationalUnit->setParent($this->parent->getEntity());
        }
        $organizationalUnit->setDescription($this->description);
        $organizationalUnit->setComments($this->comments);
        $organizationalUnit->setType($this->type);

        if ($this->networkSettings){
            $organizationalUnit->setNetworkSettings($this->networkSettings->createOrUpdateEntity($organizationalUnit->getNetworkSettings()));
        }

        return $organizationalUnit;
    }
}