<?php

namespace Functional;

use App\Entity\Client;
use App\Entity\Menu;
use App\Entity\Partition;
use App\Factory\ClientFactory;
use App\Factory\HardwareProfileFactory;
use App\Factory\MenuFactory;
use App\Factory\OrganizationalUnitFactory;
use App\Factory\PartitionFactory;
use App\Factory\UserFactory;
use App\Model\OrganizationalUnitTypes;
use App\Model\UserGroupPermissions;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Contracts\HttpClient\Exception\ClientExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\DecodingExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\RedirectionExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\ServerExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface;

class PartitionTest extends AbstractTest
{
    CONST string USER_ADMIN = 'ogadmin';
    CONST string CLIENT_CREATE = 'test-client-create';

    const string HW_PROFILE = 'HW Test';

    /**
     * @throws RedirectionExceptionInterface
     * @throws DecodingExceptionInterface
     * @throws ClientExceptionInterface
     * @throws TransportExceptionInterface
     * @throws ServerExceptionInterface
     */
    public function testGetCollectionPartitions(): void
    {
        UserFactory::createOne(['username' => self::USER_ADMIN, 'roles'=> [UserGroupPermissions::ROLE_SUPER_ADMIN]]);

        $ou = OrganizationalUnitFactory::createOne(['type' => OrganizationalUnitTypes::ORGANIZATIONAL_UNIT]);
        $hp = HardwareProfileFactory::createOne(['description' => self::HW_PROFILE]);

        $client = ClientFactory::createOne(['name' => self::CLIENT_CREATE, 'serialNumber' => '123abc', 'organizationalUnit' => $ou, 'hardwareProfile' => $hp]);

        PartitionFactory::createMany(10, ['client' => $client]);

        $this->createClientWithCredentials()->request('GET', '/partitions');
        $this->assertResponseStatusCodeSame(Response::HTTP_OK);
        $this->assertResponseHeaderSame('content-type', 'application/ld+json; charset=utf-8');
        $this->assertJsonContains([
            '@context' => '/contexts/Partition',
            '@id' => '/partitions',
            '@type' => 'hydra:Collection',
            'hydra:totalItems' => 10,
        ]);
    }

    /**
     * @throws RedirectionExceptionInterface
     * @throws DecodingExceptionInterface
     * @throws ClientExceptionInterface
     * @throws TransportExceptionInterface
     * @throws ServerExceptionInterface
     */
    public function testCreatePartition(): void
    {
        UserFactory::createOne(['username' => self::USER_ADMIN, 'roles'=> [UserGroupPermissions::ROLE_SUPER_ADMIN]]);

        $ou = OrganizationalUnitFactory::createOne(['type' => OrganizationalUnitTypes::ORGANIZATIONAL_UNIT]);
        $hp = HardwareProfileFactory::createOne(['description' => self::HW_PROFILE]);

        ClientFactory::createOne(['name' => self::CLIENT_CREATE, 'serialNumber' => '123abc', 'organizationalUnit' => $ou, 'hardwareProfile' => $hp]);
        $iri = $this->findIriBy(Client::class, ['name' => self::CLIENT_CREATE]);

        $this->createClientWithCredentials()->request('POST', '/partitions',['json' => [
            'size' => 100,
            'osName' => 'Ubuntu',
            'client' => $iri,
            'memoryUsage' => 100
        ]]);

        $this->assertResponseStatusCodeSame(201);
        $this->assertResponseHeaderSame('content-type', 'application/ld+json; charset=utf-8');
        $this->assertJsonContains([
            '@context' => '/contexts/PartitionOutput',
            '@type' => 'Partition',
            'size' => 100,
            'osName' => 'Ubuntu',
            'memoryUsage' => 100
        ]);
    }

}